'''
TeensyLC programmable joystick mouse Configuration Tool v0.1 BETA
'''

# === TkInter - pip3 install python3-tkinter
import tkinter as tk 
from tkinter import ttk
import tkinter.scrolledtext as tkst
import tkinter.font as tkFont
from tkinter import filedialog
# === PySerial - pip3 install pyserial
import serial 
import serial.tools.list_ports

'''================================================================================================
 CONSTANTS
================================================================================================'''

FLAG_NONE            = 0
FLAG_LED_ENABLED     = 1
FLAG_BEEP_ENABLED    = 2
FLAG_RUMBLE_ENABLED  = 4
FLAG_PRESET_ENABLED  = 128

ACTIONS = ["Do Nothing","Cursor move right","Cursor move left","Cursor move up","Cursor move down","Scroll vertically down",
    "Scroll vertically up","Scroll horizontally right","Scroll horizontally left","Single click button 1","Single click button 2",
    "Single click button 3","Double click button 1","Double click button 2","Double click button 3","Toggle button 1 on/off",
    "Toggle button 2 on/off","Toggle button 3 on/off","Release button 1","Release button 2","Release button 3","Hold button 1",
    "Hold button 2","Hold button 3","Swap vert/horiz scroll","Swap single/double click","Release all buttons","Next preset"]

ACTION_VALUES = [0b00000000,0b00000001,0b00000010,0b00000011,0b00000100,0b00000101,0b00000110,0b00000111,0b00001000,0b00001001,
    0b00001010,0b00001011,0b00001100,0b00001101,0b00001110,0b00001111,0b00010000,0b00010001,0b00010010,0b00010011,0b00010100,
    0b00010101,0b00010110,0b00010111,0b00011000,0b00011001,0b00011010,0b00011111]

EVENTS = ["JOY AXIS X+","JOY AXIS X-","JOY AXIS Y+","JOY AXIS Y-","JOY AXIS RECENTRE","JOY AXIS BUTTON","TOP BUTTON PRESS",
    "SCROLL BUTTON PRESS", "BOTTOM BUTTON PRESS", "MODE BUTTON PRESS", "SCROLL WHEEL-", "SCROLL WHEEL+"]

HANDLER_MASK         = 0b00011111
HANDLER_EXTRA_NONE   = 0b00000000
HANDLER_EXTRA_RUMBLE = 0b00100000
HANDLER_EXTRA_BEEP   = 0b01000000
HANDLER_EXTRA_BLINK  = 0b10000000
HANDLER_EXTRA_MASK   = 0b11100000

AXIS_X_INT_JUMP   = 0x01
AXIS_X_INT_LINEAR = 0x02
AXIS_X_INT_SMOOTH = 0x03
AXIS_Y_INT_JUMP   = 0x10
AXIS_Y_INT_LINEAR = 0x20
AXIS_Y_INT_SMOOTH = 0x30
AXIS_X_INT_MASK   = 0x0F
AXIS_Y_INT_MASK   = 0xF0

'''================================================================================================
 SETTINGS AND PRESETS CLASSES
================================================================================================'''
class Preset():
    def __init__(self):
        self.axis_x_pos = 0
        self.axis_x_neg = 0
        self.axis_y_pos = 0
        self.axis_y_neg = 0
        self.axis_recentre = 0
        self.axis_button = 0
        self.top_button = 0
        self.scroll_button = 0
        self.bottom_button = 0
        self.mode_button = 0
        self.scroll_neg = 0
        self.scroll_pos = 0
        self.bit_flags = 0
        self.axis_interpolation = 0
        self.x_axis_speed = [0,0,0,0,0,0,0,0]
        self.y_axis_speed = [0,0,0,0,0,0,0,0]
        self.scroll_vertical_rate = 0
        self.scroll_horizontal_rate = 0
        self.scroll_delay = 0

    def presetFromBytes(self, b, offset):
        self.axis_x_pos = b[offset+0]
        self.axis_x_neg = b[offset+1]
        self.axis_y_pos = b[offset+2]
        self.axis_y_neg = b[offset+3]
        self.axis_recentre = b[offset+4]
        self.axis_button = b[offset+5]
        self.top_button = b[offset+6]
        self.scroll_button = b[offset+7]
        self.bottom_button = b[offset+8]
        self.mode_button = b[offset+9]
        self.scroll_neg = b[offset+10]
        self.scroll_pos = b[offset+11]
        self.bit_flags = b[offset+12]
        self.axis_interpolation = b[offset+13]
        self.x_axis_speed = [b[offset+14],b[offset+15],b[offset+16],b[offset+17],b[offset+18],b[offset+19],b[offset+20],b[offset+21]]
        self.y_axis_speed = [b[offset+22],b[offset+23],b[offset+24],b[offset+25],b[offset+26],b[offset+27],b[offset+28],b[offset+29]]
        self.scroll_vertical_rate = b[offset+30]
        self.scroll_horizontal_rate = b[offset+31]
        self.scroll_delay = b[offset+32]

    def bytesFromPreset(self):
        b = b''
        b = b + self.axis_x_pos.to_bytes(1, byteorder='little', signed=False)
        b = b + self.axis_x_neg.to_bytes(1, byteorder='little', signed=False)
        b = b + self.axis_y_pos.to_bytes(1, byteorder='little', signed=False)
        b = b + self.axis_y_neg.to_bytes(1, byteorder='little', signed=False)
        b = b + self.axis_recentre.to_bytes(1, byteorder='little', signed=False)
        b = b + self.axis_button.to_bytes(1, byteorder='little', signed=False)
        b = b + self.top_button.to_bytes(1, byteorder='little', signed=False)
        b = b + self.scroll_button.to_bytes(1, byteorder='little', signed=False)
        b = b + self.bottom_button.to_bytes(1, byteorder='little', signed=False)
        b = b + self.mode_button.to_bytes(1, byteorder='little', signed=False)
        b = b + self.scroll_neg.to_bytes(1, byteorder='little', signed=False)
        b = b + self.scroll_pos.to_bytes(1, byteorder='little', signed=False)
        b = b + self.bit_flags.to_bytes(1, byteorder='little', signed=False)
        b = b + self.axis_interpolation.to_bytes(1, byteorder='little', signed=False)
        for a in range(0,8):
            b = b + self.x_axis_speed[a].to_bytes(1, byteorder='little', signed=False)
        for a in range(0,8):
            b = b + self.y_axis_speed[a].to_bytes(1, byteorder='little', signed=False)
        b = b + self.scroll_vertical_rate.to_bytes(1, byteorder='little', signed=False)
        b = b + self.scroll_horizontal_rate.to_bytes(1, byteorder='little', signed=False)
        b = b + self.scroll_delay.to_bytes(1, byteorder='little', signed=False)
        return b

class Settings():
    def __init__(self):
        self.current_preset = 0
        self.bit_flags = 0
        self.joy_deadzone_x = 0
        self.joy_deadzone_y = 0
        self.joy_x_min = 0
        self.joy_x_max = 0
        self.joy_y_min = 0
        self.joy_y_max = 0
        self.joy_x_centre = 0
        self.joy_y_centre = 0
        self.beep_tone = 0
        self.beep_length = 0
        self.rumble_power = 0
        self.rumble_length = 0
        self.presets = [Preset(), Preset(), Preset()]

    def settingFromBytes(self, b):
        self.current_preset = b[0]
        self.bit_flags = b[1]
        self.joy_deadzone_x = b[2] | (b[3] << 8)
        self.joy_deadzone_y = b[4] | (b[5] << 8)
        self.joy_x_min = b[6] | (b[7] << 8)
        self.joy_x_max = b[8] | (b[9] << 8)
        self.joy_y_min = b[10] | (b[11] << 8)
        self.joy_y_max = b[12] | (b[13] << 8)
        self.joy_x_centre = b[14] | (b[15] << 8)
        self.joy_y_centre = b[16] | (b[17] << 8)
        self.beep_tone = b[18] | (b[19] << 8)
        self.beep_length = b[20]
        self.rumble_power = b[21]
        self.rumble_length = b[22]
        self.presets[0].presetFromBytes(b, 24)
        self.presets[1].presetFromBytes(b, 24+33)
        self.presets[2].presetFromBytes(b, 24+33+33)

    def bytesFromSettings(self):
        b = b''
        b = b + self.current_preset.to_bytes(1, byteorder='little', signed=False)
        b = b + self.bit_flags.to_bytes(1, byteorder='little', signed=False)
        b = b + self.joy_deadzone_x.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_deadzone_y.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_x_min.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_x_max.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_y_min.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_y_max.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_x_centre.to_bytes(2, byteorder='little', signed=False)
        b = b + self.joy_y_centre.to_bytes(2, byteorder='little', signed=False)
        b = b + self.beep_tone.to_bytes(2, byteorder='little', signed=False)
        b = b + self.beep_length.to_bytes(1, byteorder='little', signed=False)
        b = b + self.rumble_power.to_bytes(1, byteorder='little', signed=False)
        b = b + self.rumble_length.to_bytes(1, byteorder='little', signed=False)
        b = b + (0).to_bytes(1, byteorder='little', signed=False) # struct alignment padding
        b = b + self.presets[0].bytesFromPreset()
        b = b + self.presets[1].bytesFromPreset()
        b = b + self.presets[2].bytesFromPreset()

        # padding to reach 126 bytes
        b = b + (0).to_bytes(1, byteorder='little', signed=False)
        b = b + (0).to_bytes(1, byteorder='little', signed=False)
        b = b + (0).to_bytes(1, byteorder='little', signed=False)

        # last 2 bytes to save (simple!) checksum
        sum = 0
        for a in range(0,126):
            sum = sum + b[a]
        b = b + sum.to_bytes(2, byteorder='little', signed=False)

        return b

'''================================================================================================
 METHODS
================================================================================================'''
def readSettings():
    if (serialPort.is_open):
        serialPort.write(b'R')
        b = bytearray(serialPort.read(128))
        settings.settingFromBytes(b)
        # dump settings hex
        #for a in range(0,128):
        #    print("{:02x} ".format(b[a]), end='')
        #print()

def writeSettings():
    if (serialPort.is_open):
        b = settings.bytesFromSettings()
        # dump settings hex
        #for a in range(0,128):
        #    print("{:02x} ".format(b[a]), end='')
        #print()
        serialPort.write(b'P')
        serialPort.write(b)
        if (serialPort.read(1) == b'K'):
            return "OK"
        else:
            return "Checksum failed. Nothing written."

def generateOnChange(obj):
    obj.tk.eval('''
        proc widget_proxy {widget widget_command args} {

            # call the real tk widget command with the real args
            set result [uplevel [linsert $args 0 $widget_command]]

            # generate the event for certain types of commands
            if {([lindex $args 0] in {insert replace delete}) ||
                ([lrange $args 0 2] == {mark set insert}) || 
                ([lrange $args 0 1] == {xview moveto}) ||
                ([lrange $args 0 1] == {xview scroll}) ||
                ([lrange $args 0 1] == {yview moveto}) ||
                ([lrange $args 0 1] == {yview scroll})} {

                event generate  $widget <<Change>> -when tail
            }

            # return the result from the real widget command
            return $result
        }
        ''')
    obj.tk.eval('''
        rename {widget} _{widget}
        interp alias {{}} ::{widget} {{}} widget_proxy {widget} _{widget}
    '''.format(widget=str(obj)))

def validateSpinbox256(newValue):
    if (len(newValue) == 0):
        return True
    if not (newValue.isdigit()):
        return False
    if (int(newValue) > 255):
        return False
    if (int(newValue) < 0):
        return False
    return True

def validateSpinbox65536(newValue):
    if (len(newValue) == 0):
        return True
    if not (newValue.isdigit()):
        return False
    if (int(newValue) > 65535):
        return False
    if (int(newValue) < 0):
        return False
    return True

'''================================================================================================
 GLOBALS
================================================================================================'''
settings = Settings()
serialPortList = [comport.device for comport in serial.tools.list_ports.comports()]
serialPort = serial.Serial()

'''================================================================================================
 USER INTERFACE
================================================================================================'''

class AccelerationFrame(tk.Frame):
    def __init__(self,master):
        tk.Frame.__init__(self,master)
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

        # validators
        vcmd256 = (self.register(validateSpinbox256),'%P')

        # settings
        self.controlFrame = tk.Frame(self)
        self.controlFrame.grid(row=0, column=0)
        self.labelMode = tk.Label(self.controlFrame,text="Interpolation:")
        self.labelMode.grid(row=0, column=0, pady=(8,8), sticky=tk.W)
        self.comboMode = ttk.Combobox(self.controlFrame, values=["Jump","Linear","Smooth-step"], state="readonly")
        self.comboMode.bind("<<ComboboxSelected>>",self.updateGraphEvent)
        self.comboMode.grid(row=0, column=1, pady=(8,8))
        self.comboMode.current(0)

        # graphics
        self.canvasFrame = tk.Frame(self)
        self.canvasFrame.grid(row=1, column=0, pady=(0,8))
        self.canvas = tk.Canvas(self.canvasFrame, width=534, height=128, bg='white')
        self.canvas.grid(row=1, column=0, columnspan=8, padx=(8,0))
        for h in range(1,7):
            self.canvas.create_line(16+(h*74),0,16+(h*74),128,fill='#DDDDDD')
        self.canvas.create_line(0,12,534,12, fill="#DDDDDD")
        self.canvas.create_text(8,100, anchor=tk.W, text="Cursor speed", angle=90)
        self.canvas.create_text(20,120, anchor=tk.W, text="Centre")
        self.canvas.create_text(500,120, anchor=tk.W, text="Max")
        self.canvas.create_text(204,120, anchor=tk.W, text="Axis distance from centre")
        self.canvas.create_line(16,0,16,128)
        self.canvas.create_line(0,112,534,112)

        self.numVal = [None,None,None,None,None,None,None,None]
        self.numVal[0] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[0].grid(row=2, column=0, sticky=tk.W, padx=(24,0))
        self.numVal[0].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[1] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[1].grid(row=2, column=1, sticky=tk.W, padx=(32,0))
        self.numVal[1].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[2] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[2].grid(row=2, column=2, sticky=tk.W, padx=(32,0))
        self.numVal[2].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[3] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[3].grid(row=2, column=3, sticky=tk.W, padx=(32,0))
        self.numVal[3].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[4] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[4].grid(row=2, column=4, sticky=tk.W, padx=(32,0))
        self.numVal[4].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[5] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[5].grid(row=2, column=5, sticky=tk.W, padx=(32,0))
        self.numVal[5].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[6] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[6].grid(row=2, column=6, sticky=tk.W, padx=(32,0))
        self.numVal[6].bind("<KeyRelease>",self.updateGraphEvent)
        self.numVal[7] = tk.Spinbox(self.canvasFrame, from_=0, to=255, width=4, validate="key", validatecommand=vcmd256, command=self.updateGraph)
        self.numVal[7].grid(row=2, column=7, sticky=tk.W, padx=(24,0))
        self.numVal[7].bind("<KeyRelease>",self.updateGraphEvent)

        self.updateGraph()

    def updateGraphEvent(self, event):
        self.updateGraph()

    def updateGraph(self):
        # don't update graph if inputs invalid
        for v in range(0,8):
            if self.numVal[v] is None:
                return
        for v in range(0,8):
            if (len(self.numVal[v].get()) == 0 or not self.numVal[v].get().isdigit()):
                return
        # redraw graph
        self.canvas.delete("graph")
        maxVal = 0
        for v in range(0,8):
            if int(self.numVal[v].get()) > maxVal:
                maxVal = int(self.numVal[v].get())
        if maxVal == 0:
            maxVal = 1
        if (self.comboMode.get() == "Jump"):
            for v in range(0,7):
                gv = 100*(int(self.numVal[v].get()) / maxVal)
                gvn = 100*(int(self.numVal[v+1].get()) / maxVal)
                self.canvas.create_line(16+(74*v),112-gv,16+(74*v)+74,112-gv, fill="#0000DD", tag="graph")
                self.canvas.create_line(16+(74*v)+74,112-gv,16+(74*v)+74,112-gvn, fill="#0000DD", tag="graph")
        elif (self.comboMode.get() == "Linear"):
            for v in range(0,7):
                gv = 100*(int(self.numVal[v].get()) / maxVal)
                gvn = 100*(int(self.numVal[v+1].get()) / maxVal)
                self.canvas.create_line(16+(74*v),112-gv,16+(74*v)+74,112-gvn, fill="#0000DD", tag="graph")
        elif (self.comboMode.get() == "Smooth-step"):
            psgv = 100*(int(self.numVal[0].get()) / maxVal)
            for v in range(0,7):
                gv = 100*(int(self.numVal[v].get()) / maxVal)
                gvn = 100*(int(self.numVal[v+1].get()) / maxVal)
                delta = gvn-gv
                for p in range(0,37):
                    percent = p/37
                    ss = self.smoothStep(0.0, 1.0, percent)
                    sgv = gv+(delta*ss)
                    self.canvas.create_line(16+(74*v+(p*2)),112-psgv,16+(74*v)+(p*2)+2,112-sgv, fill="#0000DD", tag="graph")
                    psgv = sgv
    
    def smoothStep(self, edge0, edge1, x):
        x = self.clamp((x-edge0)/(edge1-edge0), 0.0, 1.0)
        return x * x * (3-2*x)

    def clamp(self, x, lowerLimit, upperLimit):
        if x < lowerLimit:
            x = lowerLimit
        if x > upperLimit:
            x = upperLimit
        return x

'''---------------------------------------------------------------------------------------------'''
class PresetEvent():
    def __init__(self):
        self.labelEvent = None
        self.comboAction = None
        self.checkBeep = None
        self.checkVibrate = None
        self.checkBlink = None
        self.checkBeepValue = tk.BooleanVar()
        self.checkVibrateValue = tk.BooleanVar()
        self.checkBlinkValue = tk.BooleanVar()

class PresetFrame(tk.Frame):
    def __init__(self,master,preset):
        tk.Frame.__init__(self,master)
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)
        self.master = master
        self.preset = preset

        # validators
        vcmd256 = (self.register(validateSpinbox256),'%P')

        # scrollable frame
        self.canvas = tk.Canvas(self)
        self.canvas.grid(row=0, column=0, sticky=tk.NSEW)
        self.vbar = tk.Scrollbar(self, orient=tk.VERTICAL)
        self.vbar.config(command = self.canvas.yview)
        self.vbar.grid(row=0, column=1, sticky=tk.NSEW)
        self.frame = tk.Frame(self.canvas)
        self.frame_window = self.canvas.create_window((4,4), window=self.frame, anchor=tk.NW, tags="self.frame")

        # attach scrollable frame events
        self.canvas.configure(yscrollcommand=self.vbar.set)
        self.frame.bind("<Configure>", self.frame_configure)
        self.bind("<Configure>", self.self_configure)

        # add controls to scrollable frame
        self.frame.grid_rowconfigure(0, weight=1)
        self.frame.grid_columnconfigure(0, weight=1)

        # preset group
        self.groupPreset = tk.LabelFrame(self.frame, text="")
        self.groupPreset.grid(row=0, column=0, sticky=tk.NSEW, padx=(8,8), pady=(8,4))
        self.checkEnabledValue = tk.BooleanVar()
        self.checkEnabled = tk.Checkbutton(self.groupPreset, text="Enable Preset", variable=self.checkEnabledValue)
        self.checkEnabled.grid(row=0, column=0, padx=(2,2), pady=(2,2))
        if (preset == 0):
            self.checkEnabledValue.set(True)
            self.checkEnabled.config(state=tk.DISABLED)
        self.buttonLoad = tk.Button(self.groupPreset, text="Load from file", command=self.cmdLoadPresetFromFile)
        self.buttonLoad.grid(row=0, column=1, padx=(320,0))
        self.buttonSave = tk.Button(self.groupPreset, text="Save to file", command=self.cmdSavePresetToFile)
        self.buttonSave.grid(row=0, column=2)

        # event/action group
        self.groupPreset = tk.LabelFrame(self.frame, text="")
        self.groupPreset.grid(row=1, column=0, sticky=tk.NSEW, padx=(8,8), pady=(4,4))
        self.labelEvent = tk.Label(self.groupPreset, text="Event")
        self.labelEvent.grid(row=0, column=0, sticky=tk.W, padx=(0,128))
        self.labelAction = tk.Label(self.groupPreset, text="Action")
        self.labelAction.grid(row=0, column=1, sticky=tk.W, padx=(0,64))
        self.labelSpace = tk.Label(self.groupPreset, text="")
        self.labelSpace.grid(row=0, column=2, sticky=tk.EW, padx=(0,32))        
        self.labelBeep = tk.Label(self.groupPreset, text="Beep")
        self.labelBeep.grid(row=0, column=3, sticky=tk.EW, padx=(0,32))
        self.labelVibrate = tk.Label(self.groupPreset, text="Vibrate")
        self.labelVibrate.grid(row=0, column=4, sticky=tk.EW, padx=(0,16))
        self.labelBlink = tk.Label(self.groupPreset, text="Blink")
        self.labelBlink.grid(row=0, column=5, sticky=tk.EW, padx=(0,32))

        self.presetEvents = []
        for e in range(0,len(EVENTS)):
            event = PresetEvent()
            event.labelEvent = tk.Label(self.groupPreset, text=EVENTS[e])
            event.labelEvent.grid(row=e+1, column=0, sticky=tk.W)
            event.comboAction = ttk.Combobox(self.groupPreset, values=ACTIONS, state="readonly")
            event.comboAction.grid(row=e+1, column=1, sticky=tk.W)
            event.comboAction.current(0)
            event.checkBeep = tk.Checkbutton(self.groupPreset, variable=event.checkBeepValue)
            event.checkBeep.grid(row=e+1, column=3, sticky=tk.W)
            event.checkVibrate = tk.Checkbutton(self.groupPreset, variable=event.checkVibrateValue)
            event.checkVibrate.grid(row=e+1, column=4, sticky=tk.W)
            event.checkBlink = tk.Checkbutton(self.groupPreset, variable=event.checkBlinkValue)
            event.checkBlink.grid(row=e+1, column=5, sticky=tk.W)
            self.presetEvents.append(event)

        # settings group
        self.groupSettings = tk.LabelFrame(self.frame, text="")
        self.groupSettings.grid(row=2, column=0, sticky=tk.NSEW, padx=(8,8), pady=(4,4))
        
        self.labelScrollVert = tk.Label(self.groupSettings, text="Vertical Scroll Rate:")
        self.labelScrollVert.grid(row=0, column=0, sticky=tk.E, padx=(8,8), pady=(8,2))
        self.numScrollVert = tk.Spinbox(self.groupSettings, from_=0, to=65535, width=5, validate="key", validatecommand=vcmd256)
        self.numScrollVert.grid(row=0, column=1, sticky=tk.W, padx=(8,8), pady=(8,2))

        self.labelScrollHoriz = tk.Label(self.groupSettings, text="Horizontal Scroll Rate:")
        self.labelScrollHoriz.grid(row=1, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.numScrollHoriz = tk.Spinbox(self.groupSettings, from_=0, to=65535, width=5, validate="key", validatecommand=vcmd256)
        self.numScrollHoriz.grid(row=1, column=1, sticky=tk.W, padx=(8,8), pady=(2,2))

        self.labelScrollDelay = tk.Label(self.groupSettings, text="Scroll Delay Length:")
        self.labelScrollDelay.grid(row=2, column=0, sticky=tk.E, padx=(8,8), pady=(2,8))
        self.numScrollDelay = tk.Spinbox(self.groupSettings, from_=0, to=255, width=5, validate="key", validatecommand=vcmd256)
        self.numScrollDelay.grid(row=2, column=1, sticky=tk.W, padx=(8,8), pady=(2,8))

        self.labelBeeperEnabled = tk.Label(self.groupSettings, text="Beeper Enabled:")
        self.labelBeeperEnabled.grid(row=0, column=2, sticky=tk.E, padx=(128,8), pady=(8,2))
        self.checkBeeperValue = tk.BooleanVar()
        self.checkBeeperEnabled = tk.Checkbutton(self.groupSettings, variable=self.checkBeeperValue)
        self.checkBeeperEnabled.grid(row=0, column=3, sticky=tk.W, padx=(0,8), pady=(8,2))

        self.labelVibrateEnabled = tk.Label(self.groupSettings, text="Vibrator Enabled:")
        self.labelVibrateEnabled.grid(row=1, column=2, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.checkVibratorValue = tk.BooleanVar()
        self.checkVibrateEnabled = tk.Checkbutton(self.groupSettings, variable=self.checkVibratorValue)
        self.checkVibrateEnabled.grid(row=1, column=3, sticky=tk.W, padx=(0,8), pady=(2,2))

        self.labelLedsEnabled = tk.Label(self.groupSettings, text="LEDs Enabled:")
        self.labelLedsEnabled.grid(row=2, column=2, sticky=tk.E, padx=(8,8), pady=(2,8))
        self.checkLedsValue = tk.BooleanVar()
        self.checkLedsEnabled = tk.Checkbutton(self.groupSettings, variable=self.checkLedsValue)
        self.checkLedsEnabled.grid(row=2, column=3, sticky=tk.W, padx=(0,8), pady=(2,8))        

        # X acceleration group
        self.groupXAccel = tk.LabelFrame(self.frame, text="X axis acceleration")
        self.groupXAccel.grid(row=3, column=0, sticky=tk.NSEW, padx=(8,8), pady=(4,4))
        self.accelerationX = AccelerationFrame(self.groupXAccel)
        self.accelerationX.grid(row=0, column=0)

        # Y acceleration group
        self.groupYAccel = tk.LabelFrame(self.frame, text="Y axis acceleration")
        self.groupYAccel.grid(row=4, column=0, sticky=tk.NSEW, padx=(8,8), pady=(4,4))
        self.accelerationY = AccelerationFrame(self.groupYAccel)
        self.accelerationY.grid(row=0, column=0)

    def self_configure(self, event):
        # set canvas window containing frame to match containing frame
        self.canvas.itemconfigure(self.frame_window, width=self.winfo_width()-self.vbar.winfo_width())

    def frame_configure(self, event):
        # reset the scroll region to encompass the inner frame
        self.canvas.configure(scrollregion=self.canvas.bbox("all"))

    def cmdSavePresetToFile(self):
        filename = filedialog.asksaveasfilename(title="Save preset to file", defaultextension=".preset", filetypes=(("preset files","*.preset"),("All Files", "*.*")))
        if len(filename) > 0:
            try:
                presetBytes = settings.presets[self.preset].bytesFromPreset()
                file = open(filename,"wb")
                file.write(presetBytes)
                file.close()
                self.master.master.master.log("Preset written to file '" + filename + "'")
            except Exception as e:
                self.master.master.master.log("ERROR: " + str(e))

    def cmdLoadPresetFromFile(self):
        filename = filedialog.askopenfilename(title="Load preset from file", defaultextension=".preset", filetypes=(("preset files","*.preset"),("All Files", "*.*")))
        if len(filename) > 0:
            try:
                file = open(filename,"rb")
                presetBytes = file.read()
                file.close()
                settings.presets[self.preset].presetFromBytes(presetBytes, 0)
                self.updateValues()
                self.master.master.master.log("Preset read from file '" + filename + "'")
            except Exception as e:
                self.master.master.master.log("ERROR: " + str(e))

    def updateValues(self):
        if (self.preset != 0):
            self.checkEnabledValue.set((settings.presets[self.preset].bit_flags & FLAG_PRESET_ENABLED == FLAG_PRESET_ENABLED))

        self.presetEvents[0].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].axis_x_pos & HANDLER_MASK))
        self.presetEvents[1].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].axis_x_neg & HANDLER_MASK))
        self.presetEvents[2].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].axis_y_pos & HANDLER_MASK))
        self.presetEvents[3].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].axis_y_neg & HANDLER_MASK))
        self.presetEvents[4].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].axis_recentre & HANDLER_MASK))
        self.presetEvents[5].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].axis_button & HANDLER_MASK))
        self.presetEvents[6].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].top_button & HANDLER_MASK))
        self.presetEvents[7].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].scroll_button & HANDLER_MASK))
        self.presetEvents[8].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].bottom_button & HANDLER_MASK))
        self.presetEvents[9].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].mode_button & HANDLER_MASK))
        self.presetEvents[10].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].scroll_neg & HANDLER_MASK))
        self.presetEvents[11].comboAction.current(ACTION_VALUES.index(settings.presets[self.preset].scroll_pos & HANDLER_MASK))

        self.presetEvents[0].checkBeepValue.set((settings.presets[self.preset].axis_x_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[1].checkBeepValue.set((settings.presets[self.preset].axis_x_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[2].checkBeepValue.set((settings.presets[self.preset].axis_y_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[3].checkBeepValue.set((settings.presets[self.preset].axis_y_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[4].checkBeepValue.set((settings.presets[self.preset].axis_recentre & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[5].checkBeepValue.set((settings.presets[self.preset].axis_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[6].checkBeepValue.set((settings.presets[self.preset].top_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[7].checkBeepValue.set((settings.presets[self.preset].scroll_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[8].checkBeepValue.set((settings.presets[self.preset].bottom_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[9].checkBeepValue.set((settings.presets[self.preset].mode_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[10].checkBeepValue.set((settings.presets[self.preset].scroll_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))
        self.presetEvents[11].checkBeepValue.set((settings.presets[self.preset].scroll_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BEEP))

        self.presetEvents[0].checkVibrateValue.set((settings.presets[self.preset].axis_x_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[1].checkVibrateValue.set((settings.presets[self.preset].axis_x_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[2].checkVibrateValue.set((settings.presets[self.preset].axis_y_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[3].checkVibrateValue.set((settings.presets[self.preset].axis_y_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[4].checkVibrateValue.set((settings.presets[self.preset].axis_recentre & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[5].checkVibrateValue.set((settings.presets[self.preset].axis_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[6].checkVibrateValue.set((settings.presets[self.preset].top_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[7].checkVibrateValue.set((settings.presets[self.preset].scroll_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[8].checkVibrateValue.set((settings.presets[self.preset].bottom_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[9].checkVibrateValue.set((settings.presets[self.preset].mode_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[10].checkVibrateValue.set((settings.presets[self.preset].scroll_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))
        self.presetEvents[11].checkVibrateValue.set((settings.presets[self.preset].scroll_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_RUMBLE))

        self.presetEvents[0].checkBlinkValue.set((settings.presets[self.preset].axis_x_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[1].checkBlinkValue.set((settings.presets[self.preset].axis_x_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[2].checkBlinkValue.set((settings.presets[self.preset].axis_y_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[3].checkBlinkValue.set((settings.presets[self.preset].axis_y_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[4].checkBlinkValue.set((settings.presets[self.preset].axis_recentre & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[5].checkBlinkValue.set((settings.presets[self.preset].axis_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[6].checkBlinkValue.set((settings.presets[self.preset].top_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[7].checkBlinkValue.set((settings.presets[self.preset].scroll_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[8].checkBlinkValue.set((settings.presets[self.preset].bottom_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[9].checkBlinkValue.set((settings.presets[self.preset].mode_button & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[10].checkBlinkValue.set((settings.presets[self.preset].scroll_neg & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))
        self.presetEvents[11].checkBlinkValue.set((settings.presets[self.preset].scroll_pos & HANDLER_EXTRA_MASK == HANDLER_EXTRA_BLINK))

        self.numScrollVert.delete(0,len(self.numScrollVert.get()))
        self.numScrollVert.insert(0,settings.presets[self.preset].scroll_vertical_rate)
        self.numScrollHoriz.delete(0,len(self.numScrollHoriz.get()))
        self.numScrollHoriz.insert(0,settings.presets[self.preset].scroll_horizontal_rate)
        self.numScrollDelay.delete(0,len(self.numScrollDelay.get()))
        self.numScrollDelay.insert(0,settings.presets[self.preset].scroll_delay)

        self.checkLedsValue.set((settings.presets[self.preset].bit_flags & FLAG_LED_ENABLED == FLAG_LED_ENABLED))
        self.checkVibratorValue.set((settings.presets[self.preset].bit_flags & FLAG_RUMBLE_ENABLED == FLAG_RUMBLE_ENABLED))
        self.checkBeeperValue.set((settings.presets[self.preset].bit_flags & FLAG_BEEP_ENABLED == FLAG_BEEP_ENABLED))

        for a in range(0,8):
            self.accelerationX.numVal[a].delete(0,len(self.accelerationX.numVal[a].get()))
            self.accelerationX.numVal[a].insert(0,settings.presets[self.preset].x_axis_speed[a])
            self.accelerationY.numVal[a].delete(0,len(self.accelerationY.numVal[a].get()))
            self.accelerationY.numVal[a].insert(0,settings.presets[self.preset].y_axis_speed[a])

        if (settings.presets[self.preset].axis_interpolation & AXIS_X_INT_MASK == AXIS_X_INT_JUMP):
            self.accelerationX.comboMode.current(0)
        if (settings.presets[self.preset].axis_interpolation & AXIS_X_INT_MASK == AXIS_X_INT_LINEAR):
            self.accelerationX.comboMode.current(1)
        if (settings.presets[self.preset].axis_interpolation & AXIS_X_INT_MASK == AXIS_X_INT_SMOOTH):
            self.accelerationX.comboMode.current(2)

        if (settings.presets[self.preset].axis_interpolation & AXIS_Y_INT_MASK == AXIS_Y_INT_JUMP):
            self.accelerationY.comboMode.current(0)
        if (settings.presets[self.preset].axis_interpolation & AXIS_Y_INT_MASK == AXIS_Y_INT_LINEAR):
            self.accelerationY.comboMode.current(1)
        if (settings.presets[self.preset].axis_interpolation & AXIS_Y_INT_MASK == AXIS_Y_INT_SMOOTH):
            self.accelerationY.comboMode.current(2)

        self.accelerationX.updateGraph()
        self.accelerationY.updateGraph()

    def updateSettings(self):
        settings.presets[self.preset].bit_flags = (FLAG_PRESET_ENABLED if self.checkEnabledValue.get() else 0)
        settings.presets[self.preset].bit_flags = settings.presets[self.preset].bit_flags | (FLAG_LED_ENABLED if self.checkLedsValue.get() else 0)
        settings.presets[self.preset].bit_flags = settings.presets[self.preset].bit_flags | (FLAG_RUMBLE_ENABLED if self.checkVibratorValue.get() else 0)
        settings.presets[self.preset].bit_flags = settings.presets[self.preset].bit_flags | (FLAG_BEEP_ENABLED if self.checkBeeperValue.get() else 0)

        settings.presets[self.preset].axis_x_pos = ACTION_VALUES[self.presetEvents[0].comboAction.current()]
        settings.presets[self.preset].axis_x_neg = ACTION_VALUES[self.presetEvents[1].comboAction.current()]
        settings.presets[self.preset].axis_y_pos = ACTION_VALUES[self.presetEvents[2].comboAction.current()]
        settings.presets[self.preset].axis_y_neg = ACTION_VALUES[self.presetEvents[3].comboAction.current()]
        settings.presets[self.preset].axis_recentre = ACTION_VALUES[self.presetEvents[4].comboAction.current()]
        settings.presets[self.preset].axis_button = ACTION_VALUES[self.presetEvents[5].comboAction.current()]
        settings.presets[self.preset].top_button = ACTION_VALUES[self.presetEvents[6].comboAction.current()]
        settings.presets[self.preset].scroll_button = ACTION_VALUES[self.presetEvents[7].comboAction.current()]
        settings.presets[self.preset].bottom_button = ACTION_VALUES[self.presetEvents[8].comboAction.current()]
        settings.presets[self.preset].mode_button = ACTION_VALUES[self.presetEvents[9].comboAction.current()]
        settings.presets[self.preset].scroll_neg = ACTION_VALUES[self.presetEvents[10].comboAction.current()]
        settings.presets[self.preset].scroll_pos = ACTION_VALUES[self.presetEvents[11].comboAction.current()]

        settings.presets[self.preset].axis_x_pos = settings.presets[self.preset].axis_x_pos | (HANDLER_EXTRA_BEEP if self.presetEvents[0].checkBeepValue.get() else 0)
        settings.presets[self.preset].axis_x_neg = settings.presets[self.preset].axis_x_neg | (HANDLER_EXTRA_BEEP if self.presetEvents[1].checkBeepValue.get() else 0)
        settings.presets[self.preset].axis_y_pos = settings.presets[self.preset].axis_y_pos | (HANDLER_EXTRA_BEEP if self.presetEvents[2].checkBeepValue.get() else 0)
        settings.presets[self.preset].axis_y_neg = settings.presets[self.preset].axis_y_neg | (HANDLER_EXTRA_BEEP if self.presetEvents[3].checkBeepValue.get() else 0)
        settings.presets[self.preset].axis_recentre = settings.presets[self.preset].axis_recentre | (HANDLER_EXTRA_BEEP if self.presetEvents[4].checkBeepValue.get() else 0)
        settings.presets[self.preset].axis_button = settings.presets[self.preset].axis_button | (HANDLER_EXTRA_BEEP if self.presetEvents[5].checkBeepValue.get() else 0)
        settings.presets[self.preset].top_button = settings.presets[self.preset].top_button | (HANDLER_EXTRA_BEEP if self.presetEvents[6].checkBeepValue.get() else 0)
        settings.presets[self.preset].scroll_button = settings.presets[self.preset].scroll_button | (HANDLER_EXTRA_BEEP if self.presetEvents[7].checkBeepValue.get() else 0)
        settings.presets[self.preset].bottom_button = settings.presets[self.preset].bottom_button | (HANDLER_EXTRA_BEEP if self.presetEvents[8].checkBeepValue.get() else 0)
        settings.presets[self.preset].mode_button = settings.presets[self.preset].mode_button | (HANDLER_EXTRA_BEEP if self.presetEvents[9].checkBeepValue.get() else 0)
        settings.presets[self.preset].scroll_neg = settings.presets[self.preset].scroll_neg | (HANDLER_EXTRA_BEEP if self.presetEvents[10].checkBeepValue.get() else 0)
        settings.presets[self.preset].scroll_pos = settings.presets[self.preset].scroll_pos | (HANDLER_EXTRA_BEEP if self.presetEvents[11].checkBeepValue.get() else 0)

        settings.presets[self.preset].axis_x_pos = settings.presets[self.preset].axis_x_pos | (HANDLER_EXTRA_BLINK if self.presetEvents[0].checkBlinkValue.get() else 0)
        settings.presets[self.preset].axis_x_neg = settings.presets[self.preset].axis_x_neg | (HANDLER_EXTRA_BLINK if self.presetEvents[1].checkBlinkValue.get() else 0)
        settings.presets[self.preset].axis_y_pos = settings.presets[self.preset].axis_y_pos | (HANDLER_EXTRA_BLINK if self.presetEvents[2].checkBlinkValue.get() else 0)
        settings.presets[self.preset].axis_y_neg = settings.presets[self.preset].axis_y_neg | (HANDLER_EXTRA_BLINK if self.presetEvents[3].checkBlinkValue.get() else 0)
        settings.presets[self.preset].axis_recentre = settings.presets[self.preset].axis_recentre | (HANDLER_EXTRA_BLINK if self.presetEvents[4].checkBlinkValue.get() else 0)
        settings.presets[self.preset].axis_button = settings.presets[self.preset].axis_button | (HANDLER_EXTRA_BLINK if self.presetEvents[5].checkBlinkValue.get() else 0)
        settings.presets[self.preset].top_button = settings.presets[self.preset].top_button | (HANDLER_EXTRA_BLINK if self.presetEvents[6].checkBlinkValue.get() else 0)
        settings.presets[self.preset].scroll_button = settings.presets[self.preset].scroll_button | (HANDLER_EXTRA_BLINK if self.presetEvents[7].checkBlinkValue.get() else 0)
        settings.presets[self.preset].bottom_button = settings.presets[self.preset].bottom_button | (HANDLER_EXTRA_BLINK if self.presetEvents[8].checkBlinkValue.get() else 0)
        settings.presets[self.preset].mode_button = settings.presets[self.preset].mode_button | (HANDLER_EXTRA_BLINK if self.presetEvents[9].checkBlinkValue.get() else 0)
        settings.presets[self.preset].scroll_neg = settings.presets[self.preset].scroll_neg | (HANDLER_EXTRA_BLINK if self.presetEvents[10].checkBlinkValue.get() else 0)
        settings.presets[self.preset].scroll_pos = settings.presets[self.preset].scroll_pos | (HANDLER_EXTRA_BLINK if self.presetEvents[11].checkBlinkValue.get() else 0)

        settings.presets[self.preset].axis_x_pos = settings.presets[self.preset].axis_x_pos | (HANDLER_EXTRA_RUMBLE if self.presetEvents[0].checkVibrateValue.get() else 0)
        settings.presets[self.preset].axis_x_neg = settings.presets[self.preset].axis_x_neg | (HANDLER_EXTRA_RUMBLE if self.presetEvents[1].checkVibrateValue.get() else 0)
        settings.presets[self.preset].axis_y_pos = settings.presets[self.preset].axis_y_pos | (HANDLER_EXTRA_RUMBLE if self.presetEvents[2].checkVibrateValue.get() else 0)
        settings.presets[self.preset].axis_y_neg = settings.presets[self.preset].axis_y_neg | (HANDLER_EXTRA_RUMBLE if self.presetEvents[3].checkVibrateValue.get() else 0)
        settings.presets[self.preset].axis_recentre = settings.presets[self.preset].axis_recentre | (HANDLER_EXTRA_RUMBLE if self.presetEvents[4].checkVibrateValue.get() else 0)
        settings.presets[self.preset].axis_button = settings.presets[self.preset].axis_button | (HANDLER_EXTRA_RUMBLE if self.presetEvents[5].checkVibrateValue.get() else 0)
        settings.presets[self.preset].top_button = settings.presets[self.preset].top_button | (HANDLER_EXTRA_RUMBLE if self.presetEvents[6].checkVibrateValue.get() else 0)
        settings.presets[self.preset].scroll_button = settings.presets[self.preset].scroll_button | (HANDLER_EXTRA_RUMBLE if self.presetEvents[7].checkVibrateValue.get() else 0)
        settings.presets[self.preset].bottom_button = settings.presets[self.preset].bottom_button | (HANDLER_EXTRA_RUMBLE if self.presetEvents[8].checkVibrateValue.get() else 0)
        settings.presets[self.preset].mode_button = settings.presets[self.preset].mode_button | (HANDLER_EXTRA_RUMBLE if self.presetEvents[9].checkVibrateValue.get() else 0)
        settings.presets[self.preset].scroll_neg = settings.presets[self.preset].scroll_neg | (HANDLER_EXTRA_RUMBLE if self.presetEvents[10].checkVibrateValue.get() else 0)
        settings.presets[self.preset].scroll_pos = settings.presets[self.preset].scroll_pos | (HANDLER_EXTRA_RUMBLE if self.presetEvents[11].checkVibrateValue.get() else 0)

        settings.presets[self.preset].scroll_vertical_rate = int(self.numScrollVert.get())
        settings.presets[self.preset].scroll_horizontal_rate = int(self.numScrollHoriz.get())
        settings.presets[self.preset].scroll_delay = int(self.numScrollDelay.get())

        for a in range(0,8):
            settings.presets[self.preset].x_axis_speed[a] = int(self.accelerationX.numVal[a].get())
            settings.presets[self.preset].y_axis_speed[a] = int(self.accelerationY.numVal[a].get())

        axis_interpolation_x = 0
        if (self.accelerationX.comboMode.current() == 0):
            axis_interpolation_x = AXIS_X_INT_JUMP
        if (self.accelerationX.comboMode.current() == 1):
            axis_interpolation_x = AXIS_X_INT_LINEAR
        if (self.accelerationX.comboMode.current() == 2):
            axis_interpolation_x = AXIS_X_INT_SMOOTH
        axis_interpolation_y = 0
        if (self.accelerationY.comboMode.current() == 0):
            axis_interpolation_y = AXIS_Y_INT_JUMP
        if (self.accelerationY.comboMode.current() == 1):
            axis_interpolation_y = AXIS_Y_INT_LINEAR
        if (self.accelerationY.comboMode.current() == 2):
            axis_interpolation_y = AXIS_Y_INT_SMOOTH
        settings.presets[self.preset].axis_interpolation = axis_interpolation_x | axis_interpolation_y

        # if this preset is enabled, check it has a preset change action assigned
        if self.checkEnabledValue.get():
            presetChangeFound = False
            for p in range(0,12):
                if self.presetEvents[p].comboAction.current() == 27:
                    presetChangeFound = True
            if (presetChangeFound == False):
                self.master.master.master.log("WARNING: Preset " + str(self.preset+1) + " is enabled but has no 'Next preset' action assigned.")

'''---------------------------------------------------------------------------------------------'''
class SettingsFrame(tk.Frame):
    def __init__(self,master):
        tk.Frame.__init__(self,master)
        self.grid_columnconfigure(0, weight=1)
        self.master = master

        # validators
        vcmd256 = (self.register(validateSpinbox256),'%P')
        vcmd65536 = (self.register(validateSpinbox65536),'%P')

        # groups
        self.groupSettings = tk.LabelFrame(self, text="Configuration")
        self.groupSettings.grid(row=1, column=0, sticky=tk.NSEW, padx=(8,8), pady=(4,4))

        # configuration group
        self.labelJoyDeadzoneX = tk.Label(self.groupSettings, text="X Deadzone:")
        self.labelJoyDeadzoneX.grid(row=0, column=0, sticky=tk.E, padx=(8,8), pady=(8,2))
        self.numJoyDeadzoneX = tk.Spinbox(self.groupSettings, from_=0, to=65535, width=5, validate="key", validatecommand=vcmd65536)
        self.numJoyDeadzoneX.grid(row=0, column=1, sticky=tk.W, padx=(8,8), pady=(8,2))
        self.buttonCalibrate = tk.Button(self.groupSettings, text="Calibrate joystick min/max/centre", command=self.cmdCalibrateJoystick)
        self.buttonCalibrate.grid(row=0, column=3, sticky=tk.E, padx=(100,8), pady=(8,2))

        self.labelJoyDeadzoneY = tk.Label(self.groupSettings, text="Y Deadzone:")
        self.labelJoyDeadzoneY.grid(row=1, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.numJoyDeadzoneY = tk.Spinbox(self.groupSettings, from_=0, to=65535, width=5, validate="key", validatecommand=vcmd65536)
        self.numJoyDeadzoneY.grid(row=1, column=1, sticky=tk.W, padx=(8,8), pady=(2,2))

        self.labelBeeperEnabled = tk.Label(self.groupSettings, text="Beeper Enabled:")
        self.labelBeeperEnabled.grid(row=2, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.checkBeeperValue = tk.BooleanVar()
        self.checkBeeperEnabled = tk.Checkbutton(self.groupSettings, variable=self.checkBeeperValue)
        self.checkBeeperEnabled.grid(row=2, column=1, sticky=tk.W, padx=(0,8), pady=(2,2))

        self.labelBeeperTone = tk.Label(self.groupSettings, text="Beeper Tone Frequency:")
        self.labelBeeperTone.grid(row=3, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.numBeeperTone = tk.Spinbox(self.groupSettings, from_=0, to=65535, width=5, validate="key", validatecommand=vcmd65536)
        self.numBeeperTone.grid(row=3, column=1, sticky=tk.W, padx=(8,8), pady=(2,2))

        self.labelBeeperLength = tk.Label(self.groupSettings, text="Beep Length:")
        self.labelBeeperLength.grid(row=4, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.numBeeperLength = tk.Spinbox(self.groupSettings, from_=0, to=255, width=5, validate="key", validatecommand=vcmd256)
        self.numBeeperLength.grid(row=4, column=1, sticky=tk.W, padx=(8,8), pady=(2,2))

        self.labelVibrateEnabled = tk.Label(self.groupSettings, text="Vibrator Enabled:")
        self.labelVibrateEnabled.grid(row=5, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.checkVibratorValue = tk.BooleanVar()
        self.checkVibrateEnabled = tk.Checkbutton(self.groupSettings, variable=self.checkVibratorValue)
        self.checkVibrateEnabled.grid(row=5, column=1, sticky=tk.W, padx=(0,8), pady=(2,2))

        self.labelVibratePower = tk.Label(self.groupSettings, text="Vibration Power:")
        self.labelVibratePower.grid(row=6, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.numVibratePower = tk.Spinbox(self.groupSettings, from_=0, to=255, width=5, validate="key", validatecommand=vcmd256)
        self.numVibratePower.grid(row=6, column=1, sticky=tk.W, padx=(8,8), pady=(2,2))

        self.labelVibrateLength = tk.Label(self.groupSettings, text="Vibration Length:")
        self.labelVibrateLength.grid(row=7, column=0, sticky=tk.E, padx=(8,8), pady=(2,2))
        self.numVibrateLength = tk.Spinbox(self.groupSettings, from_=0, to=255, width=5, validate="key", validatecommand=vcmd256)
        self.numVibrateLength.grid(row=7, column=1, sticky=tk.W, padx=(8,8), pady=(2,2))

        self.labelLedsEnabled = tk.Label(self.groupSettings, text="LEDs Enabled:")
        self.labelLedsEnabled.grid(row=8, column=0, sticky=tk.E, padx=(8,8), pady=(2,8))
        self.checkLedsValue = tk.BooleanVar()
        self.checkLedsEnabled = tk.Checkbutton(self.groupSettings, variable=self.checkLedsValue)
        self.checkLedsEnabled.grid(row=8, column=1, sticky=tk.W, padx=(0,8), pady=(2,8))

    def cmdCalibrateJoystick(self):
        self.master.master.master.setTabState(tk.DISABLED)
        self.master.master.master.buttonSave["state"] = tk.DISABLED
        if (serialPort.is_open):
            serialPort.write(b'C')
            serialPort.read(1)
            self.master.master.master.log("Centre joystick and press the MODE button on device.")
            serialPort.read(1)
            self.master.master.master.log("Move joystick to the extreme four corners, recentre and press the MODE button on device.")
            serialPort.read(1)
            self.master.master.master.log("Calibration completed.")
            self.master.master.master.setTabState(tk.NORMAL)
            self.master.master.master.buttonSave["state"] = tk.NORMAL
            self.master.master.master.configTabs.select(0)

    def updateValues(self):
        self.numJoyDeadzoneX.delete(0,len(self.numJoyDeadzoneX.get()))
        self.numJoyDeadzoneX.insert(0,settings.joy_deadzone_x)
        self.numJoyDeadzoneY.delete(0,len(self.numJoyDeadzoneY.get()))
        self.numJoyDeadzoneY.insert(0,settings.joy_deadzone_y)
        self.numBeeperLength.delete(0,len(self.numBeeperLength.get()))
        self.numBeeperLength.insert(0,settings.beep_length)
        self.numBeeperTone.delete(0,len(self.numBeeperTone.get()))
        self.numBeeperTone.insert(0,settings.beep_tone)
        self.numVibrateLength.delete(0,len(self.numVibrateLength.get()))
        self.numVibrateLength.insert(0,settings.rumble_length)
        self.numVibratePower.delete(0,len(self.numVibratePower.get()))
        self.numVibratePower.insert(0,settings.rumble_power)
        self.checkLedsValue.set((settings.bit_flags & FLAG_LED_ENABLED == FLAG_LED_ENABLED))
        self.checkVibratorValue.set((settings.bit_flags & FLAG_RUMBLE_ENABLED == FLAG_RUMBLE_ENABLED))
        self.checkBeeperValue.set((settings.bit_flags & FLAG_BEEP_ENABLED == FLAG_BEEP_ENABLED))

    def updateSettings(self):
        settings.joy_deadzone_x = int(self.numJoyDeadzoneX.get())
        settings.joy_deadzone_y = int(self.numJoyDeadzoneY.get())
        settings.beep_length = int(self.numBeeperLength.get())
        settings.beep_tone = int(self.numBeeperTone.get())
        settings.rumble_length = int(self.numVibrateLength.get())
        settings.rumble_power = int(self.numVibratePower.get())
        settings.bit_flags = (FLAG_LED_ENABLED if self.checkLedsValue.get() else 0) 
        settings.bit_flags = settings.bit_flags | (FLAG_RUMBLE_ENABLED if self.checkVibratorValue.get() else 0) 
        settings.bit_flags = settings.bit_flags | (FLAG_BEEP_ENABLED if self.checkBeeperValue.get() else 0)

'''---------------------------------------------------------------------------------------------'''
class MainView(tk.Tk):
    def __del__(self):
        # close serial port
        if (serialPort.is_open):
            serialPort.close()

    def __init__(self):
        tk.Tk.__init__(self)

        # configure window
        self.title("Joystick Mouse Configuration Tool")
        self.geometry("640x640")
        self.resizable(0,0)
        default_font = tkFont.nametofont("TkDefaultFont")
        default_font.configure(size=8)
        self.option_add("*Font", default_font)

        # create background frame
        self.back = tk.Frame(self)
        self.back.pack_propagate(0)
        self.back.pack(fill=tk.BOTH, expand=1)
        self.back.grid_rowconfigure(1, weight=1)
        self.back.grid_columnconfigure(0, weight=1)

        # create serial chooser/connection bar
        self.serialBar = tk.Frame(self.back, pady=8)
        self.serialBar.grid(row=0, column=0, sticky=tk.NSEW)
        self.serialLabel = tk.Label(self.serialBar, text="Device Serial Port:", padx=8)
        self.serialLabel.grid(row=0, column=0, sticky=tk.NSEW)
        self.comboPort = ttk.Combobox(self.serialBar, values=serialPortList, state="readonly")
        self.comboPort.grid(row=0, column=1, sticky=tk.NSEW)
        self.buttonConnect = tk.Button(self.serialBar, text="Connect", command=self.cmdConnectSerial)
        self.buttonConnect.grid(row=0, column=2, sticky=tk.NSEW)
        self.serialLabelFill = tk.Label(self.serialBar, text="", padx=48)
        self.serialLabelFill.grid(row=0, column=3, sticky=tk.NSEW)
        self.buttonSave = tk.Button(self.serialBar, text="Send settings & presets to device", command=self.cmdUploadSettings, state=tk.DISABLED)
        self.buttonSave.grid(row=0, column=4, sticky=tk.NSEW)

        # create configuration tabs
        self.configTabs = ttk.Notebook(self.back)
        self.configTabs.grid(row=1, column=0, sticky=tk.NSEW)
        self.tabSettings = SettingsFrame(self.configTabs)
        self.configTabs.add(self.tabSettings, text="Settings", state=tk.DISABLED)
        self.tabPreset1 = PresetFrame(self.configTabs, 0)
        self.configTabs.add(self.tabPreset1, text="Preset 1", state=tk.DISABLED)
        self.tabPreset2 = PresetFrame(self.configTabs, 1)
        self.configTabs.add(self.tabPreset2, text="Preset 2", state=tk.DISABLED)
        self.tabPreset3 = PresetFrame(self.configTabs, 2)
        self.configTabs.add(self.tabPreset3, text="Preset 3", state=tk.DISABLED)

        # create text log bar
        self.logView = tkst.ScrolledText(self.back, height=10)
        self.logView.grid(row=2, column=0, sticky=tk.NSEW)

        # initialise
        self.log("To begin, select the device serial port and push the 'connect' button.")
        if len(self.comboPort["values"]) > 0:
            self.comboPort.current(0)

    def cmdConnectSerial(self):
        if len(self.comboPort["values"]) > 0:
            try:
                serialPort.port = self.comboPort.get()
                serialPort.open()
                serialPort.write(b'T')
                if (serialPort.read(1) == b'A'):
                    self.log("Connect to device: OK")
                    # read settings
                    readSettings()
                    self.updateValues()
                    self.log("Download settings: OK")
                    # enable/disable controls
                    self.setTabState(tk.NORMAL)
                    self.configTabs.select(0)
                    self.comboPort["state"] = tk.DISABLED
                    self.buttonConnect["state"] = tk.DISABLED
                    self.buttonSave["state"] = tk.NORMAL
            except Exception as e:
                self.log("ERROR: " + str(e))
                if (serialPort.is_open):
                    serialPort.close()

    def cmdUploadSettings(self):
        self.updateSettings()
        try:
            result = writeSettings()
            self.log("Upload settings: " + result)
        except Exception as e:
            self.log("ERROR: " + str(e))

    def setTabState(self, newState):
        self.configTabs.tab(0, state=newState)
        self.configTabs.tab(1, state=newState)
        self.configTabs.tab(2, state=newState)
        self.configTabs.tab(3, state=newState)

    def updateValues(self):
        self.tabSettings.updateValues()
        self.tabPreset1.updateValues()
        self.tabPreset2.updateValues()
        self.tabPreset3.updateValues()
    
    def updateSettings(self):
        self.tabSettings.updateSettings()
        self.tabPreset1.updateSettings()
        self.tabPreset2.updateSettings()
        self.tabPreset3.updateSettings()

    def log(self, text):
        self.logView.insert(tk.INSERT, " - "+text+"\n")
        self.logView.yview(tk.END)
        self.update()

if __name__ == "__main__":
    root = MainView()
    root.mainloop()
